//----------------------------------------------------------------------------
// Anti-Grain Geometry (AGG) - Version 2.5
// A high quality rendering engine for C++
// Copyright (C) 2002-2006 Maxim Shemanarev
// Contact: mcseem@antigrain.com
//          mcseemagg@yahoo.com
//          http://antigrain.com
// 
// AGG is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// AGG is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with AGG; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef AGG_EMBEDDED_RASTER_FONTS_INCLUDED
#define AGG_EMBEDDED_RASTER_FONTS_INCLUDED

#include "agg_basics.h"

namespace agg
{
    extern const int8u gse4x6[];
    extern const int8u gse4x8[];
    extern const int8u gse5x7[];
    extern const int8u gse5x9[];
    extern const int8u gse6x12[];
    extern const int8u gse6x9[];
    extern const int8u gse7x11[];
    extern const int8u gse7x11_bold[];
    extern const int8u gse7x15[];
    extern const int8u gse7x15_bold[];
    extern const int8u gse8x16[];
    extern const int8u gse8x16_bold[];
    extern const int8u mcs11_prop[];
    extern const int8u mcs11_prop_condensed[];
    extern const int8u mcs12_prop[];
    extern const int8u mcs13_prop[];
    extern const int8u mcs5x10_mono[];
    extern const int8u mcs5x11_mono[];
    extern const int8u mcs6x10_mono[];
    extern const int8u mcs6x11_mono[];
    extern const int8u mcs7x12_mono_high[];
    extern const int8u mcs7x12_mono_low[];
    extern const int8u verdana12[];
    extern const int8u verdana12_bold[];
    extern const int8u verdana13[];
    extern const int8u verdana13_bold[];
    extern const int8u verdana14[];
    extern const int8u verdana14_bold[];
    extern const int8u verdana16[];
    extern const int8u verdana16_bold[];
    extern const int8u verdana17[];
    extern const int8u verdana17_bold[];
    extern const int8u verdana18[];
    extern const int8u verdana18_bold[];
}

#endif
